@@ -476,24 +476,6 @@ def load_vi_bindings(registry, enable_visual_key=Always(),
         " Join selected lines without space. "
         event.current_buffer.join_selected_lines(separator='')
 
-    @handle('n', filter=navigation_mode)
-    def _(event):  # XXX: use `text_object`
-        """
-        Search next.
-        """
-        event.current_buffer.apply_search(
-            get_search_state(event.cli), include_current_position=False,
-            count=event.arg)
-
-    @handle('N', filter=navigation_mode)
-    def _(event):  # TODO: use `text_object`
-        """
-        Search previous.
-        """
-        event.current_buffer.apply_search(
-            ~get_search_state(event.cli), include_current_position=False,
-            count=event.arg)
-
     @handle('p', filter=navigation_mode)
     def _(event):
         """
@@ -1336,6 +1318,38 @@ def load_vi_bindings(registry, enable_visual_key=Always(),
             pos = len(b.document.text_after_cursor)
         return TextObject(pos, type=TextObjectType.LINEWISE)
 
+    @text_object('n', no_move_handler=True)
+    def _(event):
+        " Search next. "
+        buff = event.current_buffer
+        cursor_position = buff.get_search_position(
+            get_search_state(event.cli), include_current_position=False,
+            count=event.arg)
+        return TextObject(cursor_position - buff.cursor_position)
+
+    @handle('n', filter=navigation_mode)
+    def _(event):
+        " Search next in navigation mode. (This goes through the history.) "
+        event.current_buffer.apply_search(
+            get_search_state(event.cli), include_current_position=False,
+            count=event.arg)
+
+    @text_object('N', no_move_handler=True)
+    def _(event):
+        " Search previous. "
+        buff = event.current_buffer
+        cursor_position = buff.get_search_position(
+            ~get_search_state(event.cli), include_current_position=False,
+            count=event.arg)
+        return TextObject(cursor_position - buff.cursor_position)
+
+    @handle('N', filter=navigation_mode)
+    def _(event):
+        " Search previous in navigation mode. (This goes through the history.) "
+        event.current_buffer.apply_search(
+            ~get_search_state(event.cli), include_current_position=False,
+            count=event.arg)
+
     @handle('z', '+', filter=navigation_mode|selection_mode)
     @handle('z', 't', filter=navigation_mode|selection_mode)
     @handle('z', Keys.ControlJ, filter=navigation_mode|selection_mode)
@@ -1595,7 +1609,8 @@ def load_vi_bindings(registry, enable_visual_key=Always(),
 
         for p2 in buff.multiple_cursor_positions:
             text.append(original_text[p:p2])
-            if original_text[p2] == '\n':  # Don't delete across lines.
+            if p2 >= len(original_text) or original_text[p2] == '\n':
+                # Don't delete across lines.
                 p = p2
             else:
                 p = p2 + 1
@@ -1727,9 +1742,10 @@ def load_vi_search_bindings(registry, get_search_state=None,
 
     has_focus = filters.HasFocus(search_buffer_name)
     navigation_mode = ViNavigationMode()
+    selection_mode = ViSelectionMode()
     handle = create_handle_decorator(registry, filter & ViMode())
 
-    @handle('/', filter=navigation_mode)
+    @handle('/', filter=navigation_mode|selection_mode)
     @handle(Keys.ControlS, filter=~has_focus)
     def _(event):
         """
@@ -1742,7 +1758,7 @@ def load_vi_search_bindings(registry, get_search_state=None,
         # Focus search buffer.
         event.cli.push_focus(search_buffer_name)
 
-    @handle('?', filter=navigation_mode)
+    @handle('?', filter=navigation_mode|selection_mode)
     @handle(Keys.ControlR, filter=~has_focus)
     def _(event):
         """
